/**
 * Copyright (c) 2012 Marek Matula
 * All rights reserved
 */
package it.matula.expimp;

import it.matula.expimp.msoffice.exception.ReadFailed;
import it.matula.expimp.msoffice.exception.UnsupportedCellType;

import java.io.FileInputStream;
import java.io.IOException;

import org.apache.poi.openxml4j.exceptions.InvalidFormatException;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.DataFormatter;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.ss.usermodel.WorkbookFactory;
import org.junit.FixMethodOrder;
import org.junit.Test;
import org.junit.runners.MethodSorters;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Test evaluate DAYS function
 * 
 * @author mm
 * 
 */

@FixMethodOrder(MethodSorters.NAME_ASCENDING)
public class DatesTest {

	private Workbook workbook;
	private static final Logger LOGGER = LoggerFactory.getLogger(DatesTest.class);

	@Test()
	public void testDays() throws Exception {
		openFile("c:\\tmp\\days\\days_example.xlsx");
		String value = getCellContent(0, 1, 2);
		LOGGER.warn("Value: {}", value);
	}

	public void openFile(String filename) {

		Workbook w = null;
		try {
			// registerCustomFunctions();
			FileInputStream is = new FileInputStream(filename);
			w = WorkbookFactory.create(is);

			LOGGER.debug("Excel file {} loaded", filename);
		} catch (IOException | InvalidFormatException e) {

			LOGGER.error("Can't read Excel file {} ", filename);
			LOGGER.debug("Load file failed!", e);
			throw new ReadFailed(e);
		}

		this.workbook = w;
	}

	public String getCellContent(int sheet, int column, int row) {
		Cell cell = this.getCell(sheet, column, row);
		String value = getStringCellValue(sheet, column, row, cell);

		if (value != null) {
			value = value.trim();
		}
		LOGGER.trace("Cell: {} Double value: {}", sheet + "," + row + "," + column, value);
		return value;
	}

	private String getStringCellValue(int sheet, int column, int row, Cell cell) {
		String ret = "";
		if (cell != null) {
			DataFormatter df = new DataFormatter();
			int cellType = cell.getCellType();
			switch (cellType) {
			case Cell.CELL_TYPE_BOOLEAN:
			case Cell.CELL_TYPE_NUMERIC:
			case Cell.CELL_TYPE_STRING:
				ret = df.formatCellValue(cell);
				break;
			case Cell.CELL_TYPE_FORMULA:
				evaluateFormula(cell);
				ret = getStringCellValue(sheet, column, row, cell);
				break;
			case Cell.CELL_TYPE_BLANK:
				break;
			case Cell.CELL_TYPE_ERROR:
			default:
				throw new UnsupportedCellType("Sheat:" + sheet + " Column:" + column + " Row:" + row + " Cell type:" + cellType);
			}
		}
		return ret;
	}

	private void evaluateFormula(Cell cell) {
		// Force evaluation of the formula, which mutates the cell, and send it back through...
		try {
			cell.getRow().getSheet().getWorkbook().getCreationHelper().createFormulaEvaluator().evaluateInCell(cell);
		} catch (Exception e) {
			LOGGER.error("row: {} col: {}", cell.getRowIndex(), cell.getColumnIndex());
			LOGGER.error("Evaluation failed: {}", cell.getCellFormula());
			throw e;
		}
	}

	public Cell getCell(int sheet, int column, int row) {
		Cell ret = null;

		Sheet s = this.workbook.getSheetAt(sheet);
		Row r = s.getRow(row);
		if (r != null) {
			ret = r.getCell(column, Row.CREATE_NULL_AS_BLANK);
		}
		return ret;
	}

}
